@@ -1,7 +1,7 @@
 /**
  * URL Support
  *
- * Copyright (C) 2013 by
+ * Copyright (C) 2013-2014 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al.
  * This file is distributed as part of Siege
  *
@@ -435,6 +435,51 @@ url_dump(URL this)
   return;
 }
 
+URL
+url_normalize(URL req, char *location)
+{
+  URL    ret;
+  char * url;
+  size_t len = strlen(url_get_absolute(req)) + strlen(location) + 32;
+  if (strchr(location, ':') != NULL) {
+    // it's very likely normalized
+    ret = new_url(location);
+    // but we better test it...
+    if (strchr(url_get_hostname(ret), '.') != NULL) {
+      return ret;
+    }
+  }
+  if (strchr(location, '.') != NULL) {
+    // it's *maybe* host/path
+    ret = new_url(location);
+    // so we better test it...
+    if (strchr(url_get_hostname(ret), '.') != NULL) {
+      return ret;
+    }
+    // XXX: do I really need to test for localhost?
+  }
+
+  /**
+   * If we got this far we better construct it...
+   */
+  url = (char*)malloc(len);
+  memset(url, '\0', len);
+
+  if (location[0] == '/') {
+    if (strlen(location) > 1 && location[1] == '/') {
+      /* starts with // so we should use base protocol */
+      snprintf(url, len, "%s:%s", url_get_scheme_name(req), location);
+    } else {
+      snprintf(url, len, "%s://%s:%d%s", url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), location);
+    }
+  } else {
+    snprintf(url, len, "%s://%s:%d/%s", url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), location);
+  }
+  ret = new_url(url);
+  free(url);
+  return ret;
+}
+
 private void
 __url_parse(URL this, char *url)
 {
